#include <stdio.h>
#include <console.h> 
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include <limits.h>
#include <menus.h>

#include "evalutils.c"
#include "globals.cc"
#include "commands.c"

#define MAXHEADERS 21
#define MAXSAMPLES 10000
#define MAXSTATS 20
#define BIGNUM 999999
#define SIG 0.05
#define VSIG 0.01
#define ESIG 0.001
#define LOWER 1
#define UPPER 0

char STAT_LIST[MAXSTATS][MAXNAME]={"sum","sumsqr","max","min","avg","mnsqr","MLE","ssq","var","sdev","Lsig","Usig","Lvsig","Uvsig"};
char headers[MAXHEADERS][MAXNAME];
char words[MAXHEADERS][MAXNAME];
double values[MAXHEADERS][MAXSAMPLES];
double sortarray[MAXSAMPLES];
double stats[MAXHEADERS][MAXSTATS];

//prototypes (move to globals)
int chompline(FILE *fp, char line[], int lim);
int	CountWords( char *line, char words[][MAXNAME]) ;
//int comp(const DIRENTRY *rec1, const DIRENTRY *rec2);
int comp2(int val1, int val2) ;
int comp3(const void *val1,  const void *val2)  ;
void dsort(double v[], int n);

//int comp(const DIRENTRY *rec1, const DIRENTRY *rec2) { return (strcmp((char *)rec1->lname,  (char *)rec2->lname));}
int comp2(int val1, int val2) { if(val1>val2) return(val1);if(val1<val2) return(val2);else return(0);}
int comp3(const void *val1, const void *val2) { if(val1>val2) return(1);if(val1<val2) return(-1);else return(0);}
void DoStats(FILE *INFILE);
double linterp(double lowval, double highval, double index);
double CredInt(double sortedvals[], int limit, double center, double quantile, int lower);

//main
int main(int argc, char *argv[])
{
	char *recfile="record";
	FILE *RECFILE;  FILE *INFILE;
	time_t systime;

   // fprintf(fp,"\nSaturation Mutagenesis run %d, %s\n",globs.seed, ctime(&systime), time(NULL));

	RECFILE=open_append(recfile); // open files for business
	DoCommands3(stderr,argc,argv); // get commands from input line
	DoECommands(stderr,argc,argv); // get commands from input line
	INFILE=eval.file;
	DoStats(INFILE);
 	SRND(34957); // seed random number generator
	systime = time(NULL);         
	fprintf(stderr,"\nDone with EvalSat %s\n",ctime(&systime));
}

//functions

void DoStats(FILE *INFILE){
	int len,wrdcnt,test,i,j,numsamp,numcnts,mle,numstats=14, maxsort;
	char line[MAXLINE];
	double thisval, remainder, index;
	FILE *STATFILE;  FILE *COFILE;
	char *statfile="";
	char *cofile="";

	sprintf(statfile,"stats%d.out",globs.seed);
	STATFILE=open_output(statfile); // open files for business

// read first line of headers for each column
	len=chompline(INFILE,line,MAXLINE);fprintf(STATFILE,"Line: %s\n",line);
	wrdcnt=CountWords(line,headers);fprintf(STATFILE,"Words: %d\n",wrdcnt);
	
// read each subsequent line, translate words to double values
	for(j=0,numsamp=0; (len=chompline(INFILE,line,MAXLINE))!=EOF ; j++){
		wrdcnt=CountWords(line,words); //fprintf(stderr,"Words: %d\n",wrdcnt);
		for(i=0;i<wrdcnt;i++) { sscanf(words[i],"%lf",&thisval); values[i][j]=thisval;  }
		numsamp++;
	}
	
//initialize stats
	for(i=0;i<wrdcnt;i++) {
		for(j=0; j<MAXSTATS ; j++)  stats[i][j]=0.0;
		stats[i][2]=-BIGNUM;
		stats[i][3]=BIGNUM;
	}
	
// Calculate statistics
	fprintf(stderr,"Statistics. %d\n", wrdcnt);
	for(i=0,mle=3;i<wrdcnt;i++) { // for each column
		for(j=eval.burnin, numcnts=0; j<numsamp ; j++)  { // do things with each value
			stats[i][0]+=values[i][j];
			stats[i][1]+=SQR(values[i][j]);
			if ( stats[i][2] < values[i][j] ) {
				stats[i][2] = values[i][j];
				if(i==2) mle=j; // find maximum likelihood value
			}
			if ( stats[i][3] > values[i][j] ) stats[i][3] = values[i][j];
			numcnts++;
			sortarray[j-eval.burnin]=values[i][j];
		}
		
		// the basics
		stats[i][4]=stats[i][0]/numcnts; // average
		stats[i][5]=stats[i][1]/numcnts; // mean squared value
		stats[i][6]=values[i][mle]; // maxiumum likelihood value
		stats[i][7]=stats[i][1]-SQR(stats[i][0])/numcnts; 
		if (numcnts>1) stats[i][8]=stats[i][7]/(numcnts-1); // variance
		stats[i][9]=sqrt(stats[i][8]); //std deviation
		
		// sort array of values for calculating credible intervals
	 	//qsort((void *) sortarray, numcnts, sizeof(sortarray[0]),(void *)comp3);
		dsort(sortarray,numcnts);
		for (j=0; j<numcnts &&  sortarray[j] < values[i][mle]; j++) ; maxsort=j;
	 	fprintf(stderr,"%lf %d %d %d; %lf %lf\n",0.05*numcnts,(int) (0.05*numcnts),numcnts,maxsort,sortarray[maxsort],values[i][mle]); 
		stats[i][10]=CredInt(sortarray,numcnts,(double)numcnts/2.0,SIG,LOWER);
		stats[i][11]=CredInt(sortarray,numcnts,(double)numcnts/2.0,SIG,UPPER);
		stats[i][12]=CredInt(sortarray,numcnts,(double)numcnts/2.0,VSIG,LOWER);
		stats[i][13]=CredInt(sortarray,numcnts,(double)numcnts/2.0,VSIG,UPPER);
		linterp(sortarray[(int) remainder],sortarray[(int) (remainder+1)], remainder);
		fprintf(stderr, "sig %lf vsig %lf\n",SIG,VSIG); 
	}
	
// print statistics
	fprintf(STATFILE, "Maximum likelihood point %d\n", mle);
	fprintf(STATFILE, "%8s ",""); for(j=0; j<numstats ; j++) fprintf(STATFILE, "%9s ", STAT_LIST[j]); FPNF(STATFILE);
	for(i=0;i<wrdcnt;i++) { 
		fprintf(STATFILE, "%-9s ",headers[i]);
		for(j=0; j<numstats ; j++) fprintf(STATFILE, "%9.4lf ", stats[i][j]);
		FPNF(STATFILE);
	}

} // end DoStats

double CredInt(double sortedvals[], int limit, double center, double quantile, int lower){
	double index;
	int intdex;
	
	if ( (quantile < 0.0) || (quantile > 1.0) ) { fprintf(stderr, "Error 23947 %lf\n",quantile); exit(0); }
	if ( (center<=1.0) || (center >= limit) ) { fprintf(stderr, "Error 23948 %lf\n",quantile); exit(0); }
	if(lower==LOWER) index=quantile*center; else index=limit-quantile*center+1; 
	intdex=((int)(index));
	fprintf(stderr,"lim %d, cen %lf, quant %lf, lower %lf.\n", limit, center, quantile, lower);
	if( intdex <= 0) return(sortedvals[0]); else if ( (intdex +1) >= limit ) return(sortedvals[limit-1]);
	else return(  linterp(sortedvals[intdex-1],sortedvals[intdex],index)  );
}

double linterp(double lowval, double highval, double index){
	int credint;
	double  remainder;
	
 	credint=(int) index; remainder=index-credint;
 	fprintf(stderr,"low %lf, high %lf, index %lf, rem %lf, cred %d.\n", lowval, highval, index, remainder, credint);
 	if (remainder<=0.0) return(lowval);
 	else return(lowval + (highval-lowval) * remainder);
}

void dsort(double v[], int n){
	int gap, i, j;
	double temp;

	fprintf(stderr,"Yow. %d %d\n", n,n/2);

	for (gap=n/2;gap>0;gap/=2) for(i=gap;i<n;i++) for(j=i-gap;j>=0;j-=gap){
		if(v[j]<v[j+gap]) { break;}
//		fprintf(stderr,".");
		temp=v[j];
		v[j]=v[j+gap];
		v[j+gap]=temp;
	}
}


#define NOTINWORD -1
#define WORDSTART 0
int	CountWords( char *line, char words[][MAXNAME]) {
	int		i=0,numWords=0, w=-1;
static int calls=0;
	//fprintf(stderr,"Line: %s\n",line);
	while ( *line != 0 && i<MAXHEADERS && w<MAXNAME) {
		if ( ! isspace( *line ) ) {
			if ( w == NOTINWORD ) numWords++;
			words[i][++w]=*line;
		}
		else { 
			if (w != NOTINWORD ) {
				words[i][w+1]='\0';
				// fprintf(stderr,"Word: %s %d %d\n",words[i],calls,i);
				i++;
			}
			w = NOTINWORD; 
		}
		line++;
	}
	calls++; 
	//fprintf(stderr, "%d %d %d\n",i,calls,numWords);
	//if(calls>2 && i>12) {  exit(0);}
	return numWords;
}

//get next line, remove trailing carriage return
int chompline(FILE *fp, char line[], int lim){
	int c, i;
	for(i=0; (--lim>0) && ((c=getc(fp)) != EOF) && (c != '\n'); line[i++]=c);
	line[i]='\0';  if(c==EOF) return(c); else return (i);
}