#include <stdio.h>
#include <console.h> 
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include <limits.h>
#include <menus.h>

#include "simutils.c"
#include "poisson.cc"
#include "globals.cc"
//#include "windowMaker.c"
#define MAXRATES 20


//prototypes (move to globals)
void ScanRange(FILE *fpout,double counts[],int maxcount);
void SM_mcmc(FILE *fpout,double counts[],int maxcount);
void MultiRates_mcmc(double counts[],int maxcount);
double RatesLnL(double rates[MAXRATES], double rfreqs[MAXRATES], double counts[], int maxcount);
double RatesCore(int count, double rates[MAXRATES], double rfreqs[MAXRATES]);

//main
int main(int argc, char *argv[])
{
  int seed,cycles,numrates;
  int sig,maxcount,numloci,count;
  double allavg,loctotal=0,alleletotal=0;
  double counts[ALLMAX];
  double maxrate,frac2,fracburst,brstmean;
  char *recfile="record";
  char gamfile[100]; 
  char *infile="infile";
  FILE *OUTFILE;  FILE *RECFILE;  FILE *INFILE;
  

	RECFILE=open_append(recfile); // open files for business
	DoCommands3(RECFILE,argc,argv); // get commands from input line
	INFILE=globs.file;

// scan in alleles from input file
	maxcount=ScanAlleles(INFILE,counts,&alleletotal,&loctotal);
	CalcArrays(counts,maxcount,alleletotal,loctotal); // not really necessary
  	SRND(34957); // seed random number generator
  	
	InitGamma();
  	// ScanRange(RECFILE,counts,maxcount);
  	fclose(RECFILE);
	
  	if(globs.gamma) {
		gamfile[0]='\0';
		sprintf(gamfile,"gamma_%d.out",globs.seed);
		OUTFILE=open_output(gamfile); // open files for business
  		SM_mcmc(OUTFILE,counts,maxcount);
  		fclose(OUTFILE);
  	}
	if(globs.cats) MultiRates_mcmc( counts, maxcount);

	fprintf(stderr,"\nDone with SatMut.c\n");
}

//functions

void ScanRange(FILE *fpout,double counts[],int maxcount){
	double alpha,plnl,glnl,oldg,value;
	double maxa,maxi,maxg;
	double i;
	int j;
	
	fprintf(fpout,"%lf\n",gammln(alpha));
	maxg=MAXG;
	for(alpha=ASTRT;alpha<AMAX; alpha+=ASTEP){
		for(i=ISTRT;i<IMAX; i+=ISTEP){
			plnl=PoissonLnL(i,counts,maxcount);
			glnl=GammaLnL(alpha,i,counts,maxcount);
			if(glnl>maxg) { maxa=alpha; maxi=i; maxg=glnl; }
			fprintf(fpout,"%lf %lf %20.15lf %20.15lf %20.15lf\n",i,alpha, plnl,glnl,maxg);
		}
	} 
	fprintf(fpout,"\nMax alpha %lf. Max Rate %lf. Lnl at maximum was %lf.\n\n",maxa,maxi, maxg);
	fprintf(stderr,"\nMax alpha %lf. Max Rate %lf. Lnl at maximum was %lf.\n\n",maxa,maxi, maxg);	
}


void SM_mcmc(FILE *fpout,double counts[],int maxcount){
	double alpha,plnl,glnl,oldg,olda,oldi,value;
	double maxa,maxi,maxg, zero;
	double i,thisran, test;
	int movecount, j;
		
	// Markov Chain
	maxg=MAXG;
	alpha=1.0,i=1.0,oldg=maxg,movecount=0;
	for(j=0;j<globs.cycles;j++){
		glnl=GammaLnL(alpha,i,counts,maxcount);
		if(glnl>oldg)  {
		 	// fprintf(stderr, "Bigger: %lf %lf.\n", oldg, glnl);  
		 	oldg=glnl;  olda=alpha; oldi=i;}
		else { 
			thisran= ran1();
			test=exp(glnl - oldg);
			if ( thisran < test  ) { 
				// fprintf(stderr, "Smaller: %lf %lf; %lf %lf.\n", oldg, glnl, thisran, test); 
				oldg=glnl; olda=alpha; oldi=i;}
		}
		if(glnl == oldg) movecount++;
		if (j%globs.skip==0) {
			zero=GammaCore(0, olda,  oldi,  1.0);
			plnl=PoissonLnL(i,counts,maxcount);
			fprintf(fpout,"%d\t%d\t%lf\t%lf\t%lf\t%20.15lf\t%20.15lf\t%20.15lf\n",j,movecount,oldi, olda, zero, plnl, oldg, maxg);
			fprintf(stderr,"%d\n",j);
		}
		if(glnl>maxg) { maxa=alpha; maxi=i; maxg=glnl; }

//create new test parameters for the next round
		alpha=olda; i=oldi;
		alpha=alpha + ran1()*globs.amove-globs.amove/2.0;
		i=i + ran1()*globs.imove-globs.imove/2.0;
		if (alpha<0.0) alpha=-alpha;
		if (i<0.0) i= -i;
	}
	fprintf(fpout,"\nMax alpha %lf. Max Rate %lf. Lnl at maximum was %lf. %d moves out of %d.\n\n",maxa,maxi, maxg, movecount, globs.cycles);
	fprintf(stderr,"\nMax alpha %lf. Max Rate %lf. Lnl at maximum was %lf.\n\n",maxa,maxi, maxg, movecount, globs.cycles);
	
}

double movetest(double oldlnl, double newlnl);
double movetest(double oldlnl, double newlnl){
	double result, thisran, test;
	
	test=-1; thisran=-1;
	if(newlnl>oldlnl)  { result=newlnl; }
	else { 
		thisran= ran1();
		test=exp(newlnl - oldlnl);
		if ( thisran < test  ) { result=newlnl; }
	}
	// fprintf(stderr,"%lf %lf %lf %lf %lf\n", result, newlnl, oldlnl, thisran, test);
	return(result);
}

double uniMove(double oldvalue, double range);
double uniMove(double oldvalue, double range){
	double result;
	
	result = oldvalue; 
	result = result + ran1()*range-range/2.0; 
	if (result<0.0) result= -result;
	return (result);
}

double InitRates(double rates[MAXRATES], double rfreqs[MAXRATES],int numrates);
double InitRates(double rates[MAXRATES], double rfreqs[MAXRATES],int numrates){
	int i;
	for (i=0;i<MAXRATES;i++) { rates[i]=-1; rfreqs[i]=-1; }
	for (i=0;i<=numrates;i++) { rates[i]=1.0; rfreqs[i]=1.0/(double)(numrates+1); }
	fprintf(stderr,"i %d k %d.\n",i,numrates);
}

void changerates(double rates[MAXRATES], double newrates[MAXRATES], double rfreqs[MAXRATES], double newrfreqs[MAXRATES]);
void changerates(double rates[MAXRATES], double newrates[MAXRATES], double rfreqs[MAXRATES], double newrfreqs[MAXRATES]){
	int i;
	for (i=0;i<MAXRATES;i++) { rates[i]=newrates[i]; rfreqs[i]=newrfreqs[i]; }
}

void printratehead(FILE *fpout, double rates[MAXRATES]);
void printratehead(FILE *fpout, double rates[MAXRATES]){
	int i;
	
	fprintf(fpout,"%-10s\t%-10s\t","moves","cycles");
	fprintf(fpout,"%-10s\t%-10s\t%-10s\t","prob","maxprob","Pzero");
	for(i=0; i<MAXRATES, rates[i]>=0.0; i++ ){ 
		fprintf(fpout,"%-4s%-6d%-4s%-6d","rate", i, "freq", i); 
  	} fprintf(fpout, "\n");
}

void printrates(FILE *fpout, double rates[MAXRATES], double rfreqs[MAXRATES],double zero, double ml, double max, int moves, int cycles);
void printrates(FILE *fpout, double rates[MAXRATES], double rfreqs[MAXRATES],double zero, double ml, double max, int moves, int cycles){
	int i;
	
	fprintf(fpout,"%d\t%d\t",moves,cycles);
	fprintf(fpout,"%lf\t%lf\t%lf\t",ml, max, zero);
	for(i=0; i<MAXRATES, rates[i]>=0.0; i++ ){ 
		fprintf(fpout,"%lf\t%lf\t",rates[i],rfreqs[i]);
  	} fprintf(fpout, "\n");
}

void AdjustFreqs(double rfreqs[MAXRATES]);
void AdjustFreqs(double rfreqs[MAXRATES]){
	int i;
	double sum;
	
	for(i=0, sum=0.0; i<MAXRATES, rfreqs[i]>=0.0; i++ ){ 
		if (rfreqs[i]>1.0) rfreqs[i]=2.0-rfreqs[i];
		sum+=rfreqs[i];
  	} 
	for(i=0; i<MAXRATES, rfreqs[i]>=0.0; i++ ){ rfreqs[i]/=sum; } 
}

double ChangeRates(double vector[MAXRATES], int index);
double ChangeRates(double vector[MAXRATES], int index){
	double newrate, replacerate;
	int i,j,done=0;
	for(i=0; !(done || i>5) ;i++ ){
		done=1;
		newrate=uniMove(vector[index],globs.imove);
		if (index>0 && vector[index-1]>newrate) done=0;
		if (vector[index+1]>=0 && vector[index+1]<newrate) done=0;
		
		if(done==0) {
			 //fprintf(stderr, "Warning, warning 987: %d %lf %lf %lf",index,newrate,vector[index], vector[index+1]);
		}
		else { //fprintf(stderr, "G %lf %lf",vector[index], newrate); 
		}
		// if(done==0 && (index !=0 )) fprintf(stderr, "-> %lf\n", vector[index-1]); else if (done ==0) FPN;
	}
	if(i>5) newrate=vector[index];
	// fprintf(stderr, "Done: NR %lf,OR  %lf;\t*", newrate, vector[index]);
	return (newrate); 
}

void MultiRates_mcmc(double counts[],int maxcount){
	double mrlnl, oldmr, maxmr, zero;
	double rates[MAXRATES], newrates[MAXRATES], rfreqs[MAXRATES], newrfreqs[MAXRATES], maxrates[MAXRATES], maxfreqs[MAXRATES];
	int movecount, j, ratedex, k;
	char outfile[100];
	FILE *fpout;
  
	for(k=globs.cats-1,ratedex=0;k<globs.cats;k++){
		outfile[0]='\0';
	 	sprintf(outfile,"rates%d_%d.out",k+1,globs.seed);
		fpout=open_output(outfile); // open output file for business
		InitRates(rates,rfreqs,k);
		
		// Markov Chain
		maxmr=MAXG; oldmr=maxmr,movecount=0;
	 	changerates(newrates,rates,newrfreqs,rfreqs);
	 	printratehead(fpout, rates);
		for(j=0;j<globs.cycles;j++){
			mrlnl=RatesLnL(newrates, newrfreqs, counts,maxcount);
			oldmr=movetest(oldmr,mrlnl);
			if(mrlnl == oldmr) { movecount++; changerates(rates, newrates, rfreqs, newrfreqs); }
			if (j%globs.skip==0) {
				zero=RatesCore(0,  rates, rfreqs);
				printrates(fpout,rates,rfreqs,zero, oldmr, maxmr,movecount,j);
				if (j%(10*globs.skip)==0) { fprintf(stderr,"%d\n",j); }
			}
			if(mrlnl>maxmr) {  maxmr=mrlnl; changerates(maxrates,rates,maxfreqs,rfreqs);}
			newrates[ratedex]=ChangeRates(rates,ratedex);
			if (globs.varfreqs){
				newrfreqs[ratedex]=uniMove(rfreqs[ratedex],globs.fmove);
				AdjustFreqs(newrfreqs);
			}
			ratedex++; if(ratedex>k) ratedex=0;
		}
//		fprintf(fpout,"\nLnl at maximum was %lf. %d moves out of %d.\n\n", maxmr, movecount, globs.cycles);
		fprintf(stderr,"\nLnl at maximum was %lf.\n\n", maxmr, movecount, globs.cycles);
		fclose(fpout);
	}
}

void CalcArrays(double counts[], double maxcount, double alleletotal, double loctotal){
	double allavg;
	int count;
	  double pexpfreq[ALLMAX];
  double pexpcnt[ALLMAX];
  double paexpfreq[ALLMAX];
  double paexpcnt[ALLMAX];

  	allavg=alleletotal/loctotal;
  	fprintf(stderr, "Allele counts, expectations, and adjusted counts, expectations.\n");
  	for(count=0;count<maxcount;count++){ prt1(stderr,(double) ( count+1)); } FPN;
  	for(count=0;count<maxcount;count++){ prt1(stderr,counts[count]); } FPN;
  	for(count=0;count<maxcount;count++){ pexpfreq[count]=PoissonExp(allavg,count); prt1(stderr,pexpfreq[count]); } FPN;
  	for(count=0;count<maxcount;count++){ pexpcnt[count]=pexpfreq[count]*loctotal; prt1(stderr,pexpcnt[count]); } FPN;
  	for(count=0;count<maxcount;count++){ paexpfreq[count]=TruncPoiss(allavg,count); prt1(stderr,paexpfreq[count]); } FPN;
  	for(count=0;count<maxcount;count++){ paexpcnt[count]=paexpfreq[count]*loctotal; prt1(stderr,paexpcnt[count]); } FPN;
  	for(count=0;count<maxcount;count++){ prt1(stderr,paexpcnt[count]*count); } FPN;
  	fprintf (stderr,"Maximum Allele Count %d\n",maxcount);
  	fprintf (stderr,"Number of Alleles %.0lf\n",alleletotal);
  	fprintf (stderr,"Number of Loci %.0lf\n",loctotal);
  	fprintf (stderr,"Average Number alleles %lf\n\n",allavg);
}

// parse through an array of counts, and return the log probability for a particular rate
double PoissonLnL(double rate, double counts[], int maxcount){
	int count;
	double lnl, value;
	
  	for(count=0,lnl=0;count<maxcount;count++){ 
  		value=TruncPoiss(rate,count+1); 
  		if(value>0.0) { lnl+=counts[count]*log(value); }   
  	} 
  	return(lnl);
}

// Get the poisson probabilities for a set list of rates
double InitGamma(){
	int count,i;
	double rate, value, sum;
	
  	for( rate=SRATE,i=0; i<CRATES; i++,rate*=RMULT ){ 
  		porates[i]=rate; if (globs.gamma) fprintf(stderr, "Rate %lf.\n", rate);
  		for (count=0,sum=0.0;count<ALLMAX;count++){
  			poprobs[i][count] = 0.0; 
  			value = TruncPoiss(rate,count+1); 
  			if (value>0.0) poprobs[i][count] = value; 
  			sum+=poprobs[i][count];
  		//	fprintf(stderr, "\tPP rate %lf i %d  count %d PP %lf sum %lf.\n", rate, i, count,poprobs[i][count],sum );
  		}
  	} fprintf(stderr, "End rate: %lf.\n",rate);
}

double getdiff(int i){
	double rate, upper, lower, diff;

	rate=porates[i];
	if(i+1 < CRATES) upper=(rate + porates[i+1])/2.0;
	else upper=rate;
	if(i>0) lower=( rate + porates[i-1] )/2.0;
	else lower=0.0;
	diff=upper-lower; if (diff<=0.0) exit(0);
	return(diff);
}

// For an allele count, sum over the different rate categories, and rate times their probability (rfreq)
double RatesCore(int count, double rates[MAXRATES], double rfreqs[MAXRATES]){
	int i;
	double prob;
	
	for(i=0, prob=0.0; i<MAXRATES, rates[i]>=0.0; i++ ){ 
  		prob += PoissonExp(rates[i],count) * rfreqs[i]; 
  	}
	return(prob);
}

double RatesLnL(double rates[MAXRATES], double rfreqs[MAXRATES], double counts[], int maxcount){
	int count, i;
	double lnl, value, prob, thisprob, rate, upper, lower, diff, GDFsum;
	
	for(count=0,lnl=0;count<maxcount;count++){ 
		prob=RatesCore(count, rates,  rfreqs);
		if(prob>0.0) { lnl+=counts[count]*log(prob);     }
	} 
	return ( lnl );
}

// given alpha, lambda, GDFsum, calculate the gamma probability using gdf
// adjusts (normalizes) for GDFsum; poprobs must be pre-calculated
double GammaCore(int count, double alpha, double lambda, double GDFsum){
	int i;
	double prob, diff, value, rate;
	for(i=0, prob=0; i<CRATES; i++ ){ 
	  	rate=porates[i];
		diff=getdiff(i);
  		value=poprobs[i][count] * gammadf(alpha,rate/lambda)/lambda/GDFsum * diff; // use rate, normalize gdf to sum to 1
  		prob+=value;
  	}
	return (prob);
}

// parse through an array of counts, and return the log probability for a particular mean, shape
double GammaLnL(double alpha, double lambda, double counts[], int maxcount){
	int count, i;
	double lnl, value, prob, thisprob, rate, upper, lower, diff, GDFsum;
	
 	for(i=0, GDFsum=0; i<CRATES; i++ ){ 
	  	rate=porates[i];
	  	diff=getdiff(i);
	  	GDFsum+=gammadf(alpha,rate/lambda)*diff/lambda;
	  	//fprintf(stderr, " %lf %lf %lf %lf %lf %lf %lf; ", rate, alpha, lambda, gammadf(alpha,rate/lambda)*diff/lambda, gammadf(alpha,rate/lambda)/lambda, rate/lambda,diff);
	  }// FPN;
	for(count=0,lnl=0;count<maxcount;count++){ 
		prob=GammaCore(count, alpha,  lambda,  GDFsum);
//		fprintf(stderr, "P2 %lf %d %lf %lf %lf\n",prob,count, alpha,  lambda,  GDFsum);
		if(prob>0.0) { thisprob=counts[count]*log(prob); lnl+=counts[count]*log(prob);     }

	  	// fprintf(stderr, "i %d value %lf D %lf Prob %lf sum %lf cnt %lf Lnl %lf a %lf r %lf b %lf r/b %lf, gdf %lf\n",i, value, diff, thisprob, GDFsum, counts[count], lnl, alpha, rate, lambda, rate/lambda, gammadf(alpha,lambda) );
	} 
	return ( lnl );
}



void iheader(FILE *fp, char *words,char *words2,int deflt, int val){
  	fprintf(fp,"\t%s (%s): %d (%d)\n",   words, words2,deflt, val);
}

void dheader(FILE *fp, char *words,char *words2,double deflt, double val){
  	fprintf(fp,"\t%s (%s): %lf (%lf)\n",   words, words2,deflt, val);
}

void prt1(FILE *fp,double value){
	
	if ((int) value == value) fprintf(stderr,"%6.0lf ",value);
	else if (value<1.0) fprintf(stderr,"%0.4lf ",value);
	else fprintf(stderr,"%6.2lf ",value);
}

void CommandInit(){
	file_default = "infile";

	globs.cycles = cycle_default=10000; 
	globs.skip = skip_default=100; 
	globs.amove = amove_default=0.1; 
	globs.imove = imove_default=0.5; 
	globs.fmove = fmove_default=0.1; 
	globs.seed = seed_default = 9873241;
	globs.loudness = loudness_default = 0;
	globs.cats = cats_default = 0;
	globs.cline = cline_default = 0;
	globs.gamma = gamma_default = 1;
	globs.test = test_default = 1;
	globs.varfreqs = varfreqs_default = 0;
	globs.file = open_input(file_default);
}

void DoIntro(){
	fprintf(stderr,"Format: branch [skip] [cycles] [amove] [imove] [seed] [loudness]\n");
	fprintf(stderr,"Variables read in order shown; subsequent variables are optional.\n\n");
	fprintf(stderr,"Variable: Default value (chosen value): \n");
}

void ListVariables(FILE *fp,char next_file[MAXNAME]){

	time_t systime;
     systime = time(NULL);         
  //   puts(ctime(&systime));
    fprintf(fp,"\nSaturation Mutagenesis run %d, %s\n",globs.seed, ctime(&systime), time(NULL));
    fprintf(fp,"\tInput file:\t%s\t\n\n",next_file);
	iheader(fp,"Chain Length", "cycles of random # generation",  cycle_default, globs.cycles);
	iheader(fp,"Sampling Rate", "number of rate divisions",  skip_default, globs.skip);

	iheader(fp,"Gamma", "on/off? [1/0]",  gamma_default, globs.gamma);
	iheader(fp,"Categories", "Number [only if gamma off]",  cats_default, globs.cats);
	iheader(fp,"Do Frequencies", "Frequencies of categories? [even if off]",  varfreqs_default, globs.varfreqs);
	
	dheader(fp,"alpha movement","range for Markov Chain",  amove_default, globs.amove);
	dheader(fp,"rate movement","range for Markov Chain",  imove_default, globs.imove);
	dheader(fp,"frequency movement","range for Markov Chain",  fmove_default, globs.fmove);

	iheader(fp,"Test", "if test run, don't record", test_default, globs.test);
	iheader(fp,"Seed", "for random number generator", seed_default, globs.seed);
	iheader(fp,"Noise level", "debugging", loudness_default, globs.loudness);
	fprintf(fp,"\n\n");
}

void DoCommandLine(int argc, char *argv[]){
	int cr;
	cr=0; 

	globs.skip= Get_iCommand(&cr,argv,argc,  globs.skip );
	globs.cycles= Get_iCommand(&cr,argv,argc, globs.cycles  );
	globs.seed= Get_iCommand(&cr,argv,argc, globs.seed  );
	globs.test= Get_iCommand(&cr,argv,argc, globs.test  );
}

void blurb(char *name, int numb, double real);
void blurb(char *name, int numb, double real){
	fprintf(stderr, "Command read from file: %15s\t", name);
	if(numb != -1) fprintf(stderr, "%d\t", numb);
	if(real != -1) fprintf(stderr, "%lf\t", real);
	fprintf(stderr, "\n");
}

void DoCommandFile(char next_file[MAXNAME]);
void DoCommandFile(char next_file[MAXNAME]){
	int cr;  
	char c;
	char *commands="commands";
	FILE *COMM;
 	char line[500];
 	char name[20];
 	double real;
 	int number;
 	cr=0;


	fprintf(stderr, "Reading Commands.\n", name, number, real);
	COMM=open_input(commands);
	if ( COMM != NULL )
	{
		while ( (fscanf( COMM, "%s",name)) != EOF ){
			number=-1; real=-1;
			if (strcmp(name,"skip")==0) { fscanf(COMM,"%d",&number); globs.skip=number; blurb(name,number,real); }
			if (strcmp(name,"cycles")==0) { fscanf(COMM,"%d",&number); globs.cycles=number;  blurb(name,number,real);}
			if (strcmp(name,"loudness")==0) { fscanf(COMM,"%d",&number); globs.loudness=number;  blurb(name,number,real);}
			if (strcmp(name,"seed")==0) { fscanf(COMM,"%d",&number); globs.seed=number; blurb(name,number,real); }
			if (strcmp(name,"cats")==0) { fscanf(COMM,"%d",&number); globs.cats=number; }
			if (strcmp(name,"gamma")==0) { fscanf(COMM,"%d",&number); globs.gamma=number; }
			if (strcmp(name,"varfreqs")==0) { fscanf(COMM,"%d",&number); globs.varfreqs=number; }
			if (strcmp(name,"file")==0) { globs.file=GetFileName(COMM, next_file); }
			
			if (strcmp(name,"test")==0) { fscanf(COMM,"%d",&number); globs.test=number; }
			if (strcmp(name,"amove")==0) { fscanf(COMM,"%lf",&real); globs.amove=real; blurb(name,number,real); }
			if (strcmp(name,"imove")==0) { fscanf(COMM,"%lf",&real); globs.imove=real; blurb(name,number,real); }
			if (strcmp(name,"fmove")==0) { fscanf(COMM,"%lf",&real); globs.fmove=real; }
		}
		fclose( COMM );
	} FPN;
}

int ScanAlleles(FILE *INFILE, double counts[], double *alleletotal, double *loctotal){
	int count, sig, numloci, maxcount;
	*alleletotal=0; *loctotal=0;
  	for ( sig=fscanf(INFILE,"%d",&numloci),count=0;sig!=EOF;count++,sig=fscanf(INFILE,"%d",&numloci) ){
  		// fprintf (stderr,"Allele Count %d %d.\n",count+1,numloci);
  		counts[count]=(double) numloci;
  		*alleletotal+=numloci*(count+1);
  		*loctotal+=numloci;
  	} maxcount=count;
  	counts[count]=-1;
  	return(maxcount);
}

void DoCommands3(FILE *fp, int argc, char *argv[]){
	int cr;
	char next_file[40];
	FILE *REC;
	cr=0; 
	
	CommandInit();
	DoIntro();
	DoCommandFile(next_file);
	if(globs.cline) {
		// from A. Rambaut via PAML to do dialog box for command line parameterson mac.
		#ifdef __MWERKS__
			argc=ccommand(&argv);
		#endif
		DoCommandLine(argc,argv);
	}
	if(globs.test == 1) REC=fp; else REC=stderr;
	ListVariables(REC,next_file); 
}

