#include <stdio.h>
#include <console.h> 

#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include <ctype.h>
//#include "nrutil.h"

#define ITMAX 100
#define EPS 3.0e-7
#define FPMIN 1.0e-30

#define FOR(i,n) for(i=0; i<n; i++)
#define min(a,b) ((a)<(b)?(a):(b))
#define max(a,b) ((a)>(b)?(a):(b))


//float PoissonExp(double rate, int i);
float factln(int n);
float factrl(int n);
float gammln(float xx);
float gammp(float a, float x);
//float gammaf(float a, float x);
void gcf(float *gammcf, float a, float x, float *gln);
void gser(float *gamser, float a, float x, float *gln);
float gammadf(float a, float t);

void nrerror(char words[]);
// not the original function; added because I couldn't find it
void nrerror(char words[]){
	;
}

// 
float PoissonExp(double rate, int i);
float PoissonExp(double rate, int i){
  // if (i<0) nrerror("Poisson Expectation for negative numbers is undefined.");
  if(i==0) return (1.0/exp(rate));
  else return (PoissonExp(rate,i-1)*rate/i);
}

float TruncPoiss(double rate, int i);
// probability adjustment for loss of zero class
float TruncPoiss(double rate, int i){
  if(i==0) return (0.0);
  else return (  PoissonExp(rate,i)/(1.0 - PoissonExp(rate,0) )  );
}

float gammadf(float a, float t){
	return exp(-t)*pow(t,(a-1))/exp(gammln(a));
}

float factln(int n) {
  float gammln(float xx);
  void nrerror(char error_text[]);
  static float a[101];
  
  if (n < 0) nrerror("Negative factorial in routine factln");
  if (n <= 1) return 0.0;
  if (n <= 100) return a[n] ? a[n] : (a[n]=gammln(n+1.0));
  else return gammln(n+1.0);
}

float factrl(int n) {
  float gammln(float xx);
  void nrerror(char error_text[]);
  static int ntop=4;
  static float a[33]={1.0,1.0,2.0,6.0,24.0};
  int j;
  
  if (n < 0) nrerror("Negative factorial in routine factrl");
  if (n > 32) return exp(gammln(n+1.0));
  while (ntop<n) {
    j=ntop++;
    a[ntop]=a[j]*ntop;
  }
  return a[n];
}

float gammln(float xx) {
  double x,y,tmp,ser;
  static double cof[6]={76.18009172947146,-86.50532032941677,
			24.01409824083091,-1.231739572450155,
			0.1208650973866179e-2,-0.5395239384953e-5};
  int j;
  
  y=x=xx;
  tmp=x+5.5;
  tmp -= (x+0.5)*log(tmp);
  ser=1.000000000190015;
  for (j=0;j<=5;j++) ser += cof[j]/++y;
  return -tmp+log(2.5066282746310005*ser/x);
}

float gammp(float a, float x) {
  void gcf(float *gammcf, float a, float x, float *gln);
  void gser(float *gamser, float a, float x, float *gln);
  void nrerror(char error_text[]);
  float gamser,gammcf,gln;
  
  if (x < 0.0 || a <= 0.0) nrerror("Invalid arguments in routine gammp");
  if (x < (a+1.0)) {
    gser(&gamser,a,x,&gln);
    return gamser;
  } else {
    gcf(&gammcf,a,x,&gln);
    return 1.0-gammcf;
  }
}

void gcf(float *gammcf, float a, float x, float *gln) {
  float gammln(float xx);
  void nrerror(char error_text[]);
  int i;
  float an,b,c,d,del,h;
  
  *gln=gammln(a);
  b=x+1.0-a;
  c=1.0/FPMIN;
  d=1.0/b;
  h=d;
  for (i=1;i<=ITMAX;i++) {
    an = -i*(i-a);
    b += 2.0;
    d=an*d+b;
    if (fabs(d) < FPMIN) d=FPMIN;
    c=b+an/c;
    if (fabs(c) < FPMIN) c=FPMIN;
    d=1.0/d;
    del=d*c;
    h *= del;
    if (fabs(del-1.0) < EPS) break;
  }
  if (i > ITMAX) nrerror("a too large, ITMAX too small in gcf");
  *gammcf=exp(-x+a*log(x)-(*gln))*h;
}

void gser(float *gamser, float a, float x, float *gln) {
  float gammln(float xx);
  void nrerror(char error_text[]);
  int n;
  float sum,del,ap;
  
  *gln=gammln(a);
  if (x <= 0.0) {
    if (x < 0.0) nrerror("x less than 0 in routine gser");
    *gamser=0.0;
    return;
  } else {
    ap=a;
    del=sum=1.0/a;
    for (n=1;n<=ITMAX;n++) {
      ++ap;
      del *= x/ap;
      sum += del;
      if (fabs(del) < fabs(sum)*EPS) {
	*gamser=sum*exp(-x+a*log(x)-(*gln));
	return;
      }
    }
    nrerror("a too large, ITMAX too small in routine gser");
    return;
  }
}

#undef ITMAX
#undef EPS
#undef FPMIN
