#include <stdio.h>
#include <console.h> 

#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include <ctype.h>


#define RESIDUES 20
#define min2(A,B) ((A) < (B) ? (A) : (B))
#define min3(A,B,C) ((A) < (B) ? min2(A,C) : min2(B,C)) 
#define max2(A,B) ((A) > (B) ? (A) : (B))
#define max3(A,B,C) ((A) > (B) ? max2(A,C) : max2(B,C))
#define SMALLNUM 1.0e-20 /* a number slightly greater than 0 */

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS2 1.2e-7
#define RNMX (1.0-EPS2)
#define FPN fprintf(stderr,"\n")


static long idummy;

float ran1();
float ran1(){
	int j; long k;
	static long iy=0;
	static long iv[NTAB];
	float temp;
	long *idum;
	
	if (idummy==NULL) idummy=-2934;
	idum = &idummy;
	if ( (*idum <= 0) || !iy){
		if (-(*idum) <1) *idum=1;
		else *idum = -(*idum);
		for (j=NTAB+7; j>=0;j--){
			k=(*idum/IQ);
			*idum=IA*(*idum-k+IQ)-IR*k;
			if (*idum < 0 ) *idum += IM;
			if (j < NTAB ) iv[j] = *idum;
		}
		iy=iv[0];
	}
	k=(*idum) / IQ;
	*idum=IA*(*idum-k*IQ)-IR*k;
	if (*idum < 0) *idum += IM;
	j = iy/NDIV;
	iy=iv[j];
	iv[j]= *idum;
	if ((temp=AM*iy) > RNMX ) return RNMX;
	else return temp;						
}


#define RANMAX 2147483647.0
#define FRND ran1()
#define SRND(x) (idummy=-x)
//#define SRND(x) ran1(x)
#define SQR(x) ((x)*(x))

//#define FRND (double)((double)random()/RANMAX)

#define Max_InTaxa 4096
/* #define Max_InTaxa 16384  for 14 splits */
/* #define Max_OutTaxa 4096 */
#define Max_OutTaxa 150
#define Max_Epochs 14 
#define MAXNAME 30 
#define MAXLINE 256 

#define Max_Length 500
#define Max_Trees 2
#define MAXFILES 10

int NumSeqs;
char File_Names[MAXFILES][MAXNAME];
double AllEpochs[MAXFILES];
int AllSeqLengths[MAXFILES];
int AllSeqs[MAXFILES][Max_OutTaxa][Max_Length];
/* int Sequences[Max_OutTaxa][Max_Length];
char SeqNames[Max_OutTaxa][MAXNAME];
 */
int SeqLength;

int Seqs_Needed[Max_InTaxa];
int Res_Selected[Max_Length];
int File_Selected[Max_Length];
int Corr_Sites[Max_Length];
int Trees[Max_Trees][Max_OutTaxa];

char AA_List[RESIDUES]={'G','A','V','L','I','M','F','P','S','T','C','N','Q','Y','W','D','E','H','K','R'};

int Get_iCommand();
double Get_dCommand();
void Get_Dayhoff();
 int frombase(char basechar);  
int Poisson();
int power();
double exp();
double log ();
double floor();
double pow();
long random();
double sqrt();

//declarations

typedef struct CorrPair_  
/* contains necessary information to define a correlated pair:
   alternate residues at each position, frequency at first position,
   frequency at second position depending on residue at first
   position, and rates at second position (dependent again) relative
   to rate at first site.  */
{
  int res1a;
  int res1b;
  double freq1;
  int res2a;
  int res2b;
  double freq2a;
  double freq2b;
  double rate2a;		/* rate relative  to site 1 */
  double rate2b;
  
}  CPair_;


int  frombase(char basechar) 
/* take character code for an amino acid and return integer code */
{   /* Return integer index of Amino Acid list for the base indicated. */
  int whichamino;
  
  for(whichamino=0;AA_List[whichamino]!=basechar && whichamino<RESIDUES;whichamino++) ;	
  return(whichamino);
}

int power(int x, int y)
{
  if(y<1)return(1);
  else if (y<16) return(x*power(x,y-1));
  else return (x*power(x,15));
}

FILE *open_input(char *filename);
FILE *open_input(char *filename) {     /* open filename for input */
  FILE *f;
  f = fopen(filename, "r");
  if (f == 0) {
    printf("can't open '%s', peabrain\n", filename);
    exit(0);
  }
  return f;
}

FILE *open_output(char *filename);
FILE *open_output(char *filename) {     /* open filename for input */
  FILE *f;
  f = fopen(filename, "w");
  if (f == 0) {
    printf("can't open '%s', peabrain\n", filename);
    exit(0);
  }
  return f;
}

FILE *open_append(char *filename) ;    
FILE *open_append(char *filename) {     /* open filename for input */
  FILE *f;
  f = fopen(filename, "a");
  if (f == 0) {
    printf("can't open '%s', peasoupbrain\n", filename);
    exit(0);
  }
  return f;
}

// Open for reading the file named as input.
FILE *GetFileName(FILE *fp, char next_file[MAXNAME]);
FILE *GetFileName(FILE *fp, char next_file[MAXNAME])
{
  int noiselevel;
  FILE *f;

  noiselevel=0;
  fscanf(fp,"%s", next_file);
  f=(fopen(next_file,"r"));
 // if (loudness >noiselevel) fprintf(stderr,"%s, %d",next_file,f);
  if (f == 0) {
    printf("can't open '%s', peabrain\n", next_file);
    exit(0);
  }
  return f;
}

int  Get_iCommand(int *argument,char *arglist[], int listlength, int the_default) {
  int new_command;
  
  (*argument)++;
  if (listlength > *argument) sscanf(arglist[*argument],"%d", &new_command);
  else new_command=the_default;
  return(new_command);
}

double  Get_dCommand(int *argument, char *arglist[], int listlength, double the_default) {
  double new_command;
  
  (*argument)++;
  if (listlength > *argument) sscanf(arglist[*argument],"%lf", &new_command);
  else new_command=the_default;
  return(new_command);
}

int Poisson(double lambda) {
    int n;
    double mean,tmp;
    n = 0;
    tmp = FRND;
    mean = exp(0.0-lambda);
    while (tmp > mean)
	{
	    tmp *= FRND;
	    n++;
	}
    return(n);
}

     /* Read and Set-up the PAM or PAM-like matrix */
void Get_Dayhoff(double **evec,double **ievec, double *lneval, double *bfreqs) {
  extern int frombase(char basechar);
  extern int *ivec(int high);
  extern void freeivec(int *vecname);
  
  FILE *reveqps,*evalfile,*evecfile,*ievecfile;
  int r,s,*order;
   char thisamino;
  
  reveqps=fopen("reveqps.day","r");
  evalfile=fopen("evalfl.day","r");
  ievecfile= fopen("ievecfl.day","r");
  evecfile= fopen("evecfl.day","r");

  r=0;
  while (r<RESIDUES)    {
    thisamino=getc(reveqps);
    if (frombase(thisamino) < RESIDUES){
      fscanf(reveqps,"%lf",&bfreqs[frombase(thisamino)]);
      r++;
    }
  } /* while r */
  bfreqs[RESIDUES]=1.0; /* This is the "Unknown" amino acid */
  fclose(reveqps);

  r=0;
  while (r<RESIDUES)   {
    thisamino=getc(evalfile);
    if (frombase(thisamino) < RESIDUES)      {
      fscanf(evalfile,"%lf",&lneval[frombase(thisamino)]);
      lneval[frombase(thisamino)]=
	log(lneval[frombase(thisamino)]);
      r++;
    } /* if */
  } /* fwhile r */
  fclose(evalfile);
  
  order=ivec(RESIDUES-1);
  for (r=0;r< RESIDUES;r++)    {  
    thisamino= getc(evecfile);
    while (thisamino== ' ')
      thisamino=getc(evecfile);
    order[r]= frombase(thisamino);
  }  
  for (r=0;r< RESIDUES;r++)   {  
    for (s=0;s < RESIDUES;s++)
      fscanf(evecfile,"%lf", &evec[order[r]][order[s]]);
  }
  fclose(evecfile);
  
  for (r=0;r< RESIDUES;r++)    {  
    thisamino= getc(ievecfile);
    while (thisamino== ' ')
         thisamino=getc(ievecfile);
    order[r]= frombase(thisamino);
  }
  for (r=0;r< RESIDUES;r++)   
    for (s=0;s < RESIDUES;s++)
      fscanf(ievecfile,"%lf",&ievec[order[r]][order[s]]);
  fclose(ievecfile);
  
  freeivec(order);
}


